@@ -30,7 +30,7 @@
 # @authors Marcus Lechner, Clemens Elias Thoelken
 # @email lechner@staff.uni-marburg.de
 # @company University of Maruburg
-# @date 2016-08-26
+# @date 2017-04-18
 #
 ##########################################################################################
 
@@ -47,7 +47,7 @@ use Thread::Queue;
 ##########################################################################################
 # Variables
 ##########################################################################################
-our $version = "5.15";
+our $version = "5.16";
 our $step = 0;		# 0/1/2/3	-> do all / only apply step 1 / only apply step 2 / only apply step 3
 our $verbose = 1;	# 0/1		-> don't / be verbose
 our $debug = 0;		# 0/1		-> don't / show debug data
@@ -60,14 +60,15 @@ our $alpha = 0.5;	# Alpha value for ffadj_mcs.py
 our $connectivity = 0.1;# Algebr. connectivity threshold
 our $cpus   = 0;	# 0 = autodetect
 our $evalue = "1e-05";
+our $purity = 0.75;
 our $coverage = 50;	# Percent coverage threshold for two proteins
 our $identity = 25;	# Percent identity threshold for two proteins
 our $blastmode = "blastp+";
 #our $tmpdir = "./";	# Dir for tmp-files
 our $sim = 0.95;
 our $report = 3;
-our $startat = 0;
-our $stopat = -1;
+our $startat = undef;	# removed 5.16
+our $stopat = undef;	# removed 5.16
 our $keep = 0;
 our $force = 0;
 our $selfblast = 0;
@@ -98,6 +99,12 @@ our $run_id = "";
 our %gene_counter;		# Holds the number of genes for each data file (for sorting)
 our $threads_per_process :shared = 1;	# Number of subthreads for blast
 
+# Split work
+our $split_to_X_jobs = -1;
+our $jobnumber = -1;
+our $part = -1;
+
+
 ##########################################################################################
 # Parameters
 ##########################################################################################
@@ -117,6 +124,7 @@ foreach my $option (@ARGV) {
   elsif ($option =~ m/^--?cpus=(\d*)$/)    		{ $cpus = $1; }
   elsif ($option =~ m/^--?cpus=auto$/)	     		{ $cpus = 0; }
   elsif ($option =~ m/^--?alpha=([0-9\.]+)$/)    	{ $alpha = $1; }
+  elsif ($option =~ m/^--?purity=([0-9\.]+)$/)    	{ $purity = $1; }
   elsif ($option =~ m/^--?report=([0-9]+)$/)   		{ $report = $1; }  
   elsif ($option =~ m/^--?conn=([0-9\.]+)$/)   		{ $connectivity = $1; }
   elsif ($option =~ m/^--?cov=([0-9]+)$/)    		{ $coverage = $1; }
@@ -126,6 +134,7 @@ foreach my $option (@ARGV) {
   elsif ($option =~ m/^--?sim=([0-9\.]+)$/)    		{ $sim = $1; }
   elsif ($option =~ m/^--?startat=([0-9]+)$/)    	{ $startat = $1; }
   elsif ($option =~ m/^--?stopat=([0-9]+)$/)    	{ $stopat = $1; }
+  elsif ($option =~ m/^--?jobs?=([\d]+)\/([\d]+)$/) 	{ $jobnumber = $1; $split_to_X_jobs = $2; }  
   elsif ($option =~ m/^--?selfblast$/) 			{ $selfblast = 1; }  
   elsif ($option =~ m/^--?selfblast=(0|1)$/) 		{ $selfblast = $1; }  
   elsif ($option =~ m/^--?singles$/) 			{ $singles = 1; }  
@@ -145,7 +154,7 @@ foreach my $option (@ARGV) {
   elsif ($option =~ m/^--?graph/)			{ $nograph = 0; }  
   elsif ($option =~ m/^--?desc/)			{ $desc = 1; }  
   elsif ($option =~ m/^--?project=(.*)$/)		{ $project = $1; }
-  elsif ($option =~ m/^--?blastParameters=(.*)$/i)	{ $blastOptions = $1;}
+  elsif ($option =~ m/^--?subpara=(.*)$/i)	{ $blastOptions = $1;}
   elsif ($option !~ /^-/)				{ push(@files,$option); }
   else  {&print_usage(); die "Invalid command line option: \'$option\'!\n"; }
 }
@@ -153,13 +162,23 @@ foreach my $option (@ARGV) {
 ##########################################################################################
 # Check parameters
 ##########################################################################################
-if ($startat != 0 || $stopat != -1) {
-	if ($step != 2) {
-		&Error("Parameters -startat and -stopat only work for step 2!");
-	}
-	$run_id = "_$startat-$stopat";
+if (defined($startat) || defined($stopat)) {
+	&Error("Sorry, -startat and -stopat were removed. Please use -jobs=M/N for more flexible job splitting.");	
 }
 
+if ($split_to_X_jobs == 0) {
+	&Error("Job parameter use incorrectly. Number of jobs cannot be 0. Valid format: M/N");
+}
+
+if ($jobnumber != -1 && ($jobnumber == 0 || $jobnumber > $split_to_X_jobs)) {
+	&Error("Job parameter use incorrectly. Job number cannot be 0 or greater than number of jobs. Valid format: M/N");
+}
+if ($jobnumber != -1) {
+	if ($step != 2) {&Error("Parameter -jobs only works for step 2!");}
+	$run_id = "_$jobnumber"."_".$split_to_X_jobs;
+}
+
+
 our $simgraph = "$project.blast-graph$run_id";		# Output file graph
 our $syngraph = "$project.ffadj-graph$run_id";		# Output file synteny
 
@@ -175,7 +194,7 @@ our $desctable = "$project.descriptions";			# Output file seq descriptions
 
 if (-e "$project.proteinortho") {
 	print STDERR "!!!\nWarning: Data files for project '$project' already exists. Previous output files might be overwritten.\n";
-	print STDERR "Press CTRL + C to prevent me from proceeding)\nWaiting for 10 seconds...\n!!!\n";
+	print STDERR "Press CTRL + C to prevent me from proceeding\nWaiting for 10 seconds...\n!!!\n";
 	sleep(10);
 	print STDERR "\nWell then, proceeding...\n\n";
 }
@@ -190,7 +209,7 @@ if (-e "$project.proteinortho") {
 # Always do
 &check_files;		# Check files, count sequences
 @files = ();
-foreach my $file (sort { $gene_counter{$b} <=> $gene_counter{$a} } keys %gene_counter) {push(@files,$file);}	# Biggest first
+foreach my $file (sort { if ($gene_counter{$a} == $gene_counter{$b}) {$a cmp $b;} else {$gene_counter{$b} <=> $gene_counter{$a};} } keys %gene_counter) {push(@files,$file);}	# Biggest first # Alphabet otherwise 5.16
 
 
 # Step 1, check files and make indices
@@ -246,36 +265,36 @@ sub clean {
 
 sub cluster {
 	print STDERR "Clustering by similarity (Proteinortho mode)\n";
-	system("$po_path/proteinortho5_clustering -verbose $verbose -conn $connectivity -rmgraph '$rm_simgraph' $simgraph* >'$simtable'");
+	system("$po_path/proteinortho5_clustering -verbose $verbose -conn $connectivity -purity $purity -rmgraph '$rm_simgraph' $simgraph* >'$simtable'");
 	if ($singles) {
 		print STDERR "Adding singles...\n";
 		my $fastas = "'".join("' '",@files)."'";
 		system("$po_path/proteinortho5_singletons.pl $fastas <'$simtable' >>'$simtable'");
 	}
-        print STDERR "-> Output written to $simtable\n";
+        print STDERR "[OUTPUT] -> written to $simtable\n";
 	unless ($nograph) {
 		print STDERR "Writing graph...\n";
 #		system("$po_path/proteinortho5_clean_edges -e '$rm_simgraph' $simgraph* >'$csimgraph'");
 		system("$po_path/proteinortho5_clean_edges2.pl '$rm_simgraph' $simgraph* >'$csimgraph'");
 		unless ($keep) {unlink($rm_simgraph);}
-		print STDERR "-> Output written to $csimgraph\n";
+		print STDERR "[OUTPUT] -> written to $csimgraph\n";
 	}
 
 	if ($synteny) {
 		print STDERR "\nClustering by gene-order (POFF mode)\n";
-		system("$po_path/proteinortho5_clustering -verbose $verbose -conn $connectivity -rmgraph '$rm_syngraph' $syngraph* >'$syntable'");
+		system("$po_path/proteinortho5_clustering -verbose $verbose -conn $connectivity -purity $purity -rmgraph '$rm_syngraph' $syngraph* >'$syntable'");
 		if ($singles) {
 		print STDERR "Adding singles...\n";
 			my $fastas = "'".join("' '",@files)."'";
 			system("$po_path/proteinortho5_singletons.pl $fastas <'$syntable' >>'$syntable'");
 		}
-		print STDERR "-> Output written to $syntable\n";
+		print STDERR "[OUTPUT] -> written to $syntable\n";
 		unless ($nograph) {
 			print STDERR "Writing graph...\n";
 #			system("$po_path/proteinortho5_clean_edges -e '$rm_syngraph' $syngraph* >'$csyngraph'");
 			system("$po_path/proteinortho5_clean_edges2.pl '$rm_syngraph' $syngraph* >'$csyngraph'");
 			unless ($keep) {unlink($rm_syngraph);}
-			print STDERR "-> Output written to $csyngraph\n";
+			print STDERR "[OUTPUT] -> written to $csyngraph\n";
 		}
 	}
 }
@@ -290,11 +309,32 @@ sub print_usage {
 print STDERR << "JUS";
 
 Usage: proteinortho5.pl [OPTIONS] FASTA1 FASTA2 [FASTA...]
-Options: -e=          E-value for blast [default: 1e-05]
-         -p=          blast program [default: blastp+]
-                      {blastn|blastp|tblastx|blastn+|blastp+|tblastx+}
+Options:
+         [General options]
          -project=    prefix for all result file names [default: myproject]
+         -cpus=       number of processors to use [default: auto]
+         -verbose     keeps you informed about the progress
          -temp=       path for temporary files [default: working directory]
+         -keep        stores temporary blast results for reuse
+         -force       forces recalculation of blast results in any case
+         -clean       remove all unnecessary files after processing
+         -step=       1 -> generate indices
+                      2 -> run blast (and ff-adj, if -synteny is set)
+                      3 -> clustering
+                      0 -> all (default)
+
+         [Search options]
+         -p=          blast program [default: blastp+]
+         -e=          E-value for blast [default: 1e-05]
+                      {blastn|blastp|tblastx|blastn+|blastp+|tblastx+}
+         -selfblast   apply selfblast, detects paralogs without orthologs
+         -sim=        min. similarity for additional hits (0..1) [default: 0.95]
+         -identity=   min. percent identity of best blast hits [default: 25]
+         -cov=        min. coverage of best blast alignments in % [default: 50]
+         -subpara=    additional parameters for the search tool
+                      example -subpara='-seg no'
+
+         [Synteny options]
          -synteny     activate PoFF extension to separate similar sequences
                       by contextual adjacencies (requires .gff for each .fasta)
          -dups=       PoFF: number of reiterations for adjacencies heuristic,
@@ -303,33 +343,30 @@ Options: -e=          E-value for blast [default: 1e-05]
                       adjacency matches (default: 3)
          -alpha=      PoFF: weight of adjacencies vs. sequence similarity
                       (default: 0.5)
-         -desc        write description files (for NCBI FASTA input only)
-         -keep        stores temporary blast results for reuse
-         -force       forces recalculation of blast results in any case
-         -cpus=       number of processors to use [default: auto]
-         -selfblast   apply selfblast, detects paralogs without orthologs
+
+         [Clustering options]
          -singles     report singleton genes without any hit
-         -identity=   min. percent identity of best blast hits [default: 25]
-         -cov=        min. coverage of best blast alignments in % [default: 50]
+         -purity=     avoid spurious graph assignments [0.01-1, default 0.75]
          -conn=       min. algebraic connectivity [default: 0.1]
-         -sim=        min. similarity for additional hits (0..1) [default: 0.95]
-         -step=       1 -> generate indices
-                      2 -> run blast (and ff-adj, if -synteny is set)
-                      3 -> clustering
-                      0 -> all (default)
+         -nograph     do not generate .graph file (pairwise orthology relations)
+
+         [Misc options]
+         -desc        write description files (for NCBI FASTA input only)
          -blastpath=  path to your local blast (if not installed globally)
-         -verbose     keeps you informed about the progress
-         -clean       remove all unnecessary files after processing
-         -graph       generate .graph files (pairwise orthology relations)
          -debug       gives detailed information for bug tracking
 
-         More specific blast parameters can be defined by
-         -blastParameters='[parameters]' (e.g. -blastParameters='-seg no')
-
-         In case jobs should be distributed onto several machines, use
-         -startat=    File number to start with (default: 0)
-         -stopat=     File number to end with (default: -1)
+         [Large compute jobs]
+          In case jobs should be distributed onto several machines, use
+         -jobs=M/N    N defines the number of defined job groups (e.g. PCs)
+                      M defines the set of jobs to run in this process
 
+         Example:     run with 
+                        -step=1
+                      to prepare data then to split a run on two machines use
+                        -jobs=1/2 -step=2 on PC one and
+                        -jobs=2/2 -step=2 on PC two
+                      finally run with
+                        -step=3 to finalize
 JUS
 }
 
@@ -373,7 +410,23 @@ sub set_threads_per_process {
 sub run_blast {
 	# Jobs todo
 	$jobs_todo = 0;
-	for (my $k = $startat; $k < scalar(@files); $k++) {$jobs_todo += scalar(@files)-1-$k+$selfblast;if ($stopat > -1 && $stopat <= $k) {last;}}
+	for (my $i = 0; $i < scalar(@files)-1+$selfblast; $i++) {$jobs_todo += scalar(@files)-$i-1+$selfblast;}
+
+	# Divide 5.16
+	$part = int($jobs_todo/$split_to_X_jobs);	# Round up to not miss anything
+#	print STDERR "$jobs_todo/$split_to_X_jobs = $part\n";
+	my $from = ($jobnumber-1)*($part)+1;
+	if ($jobnumber == 1) {$from = 1;}
+	my $to = ($jobnumber-1)*($part)+$part;
+	if ($jobnumber == $split_to_X_jobs) {$to = $jobs_todo;}
+
+	$part = 1+$to-$from;	# real part
+
+	if ($split_to_X_jobs != -1 && $part < 1) {&Error("I have problems coordinating $split_to_X_jobs groups for $jobs_todo jobs.");}
+	if ($split_to_X_jobs <= 1) {$from = -1; $to = -1; $split_to_X_jobs = -1; $part = -1;}
+
+#	print STDERR "$from - $to (TODO: $jobs_todo)\n";
+
 	&set_threads_per_process(0);	# Check if we can apply more threads, nothing is running so far
 	&print_blast_stats();
 
@@ -381,26 +434,26 @@ sub run_blast {
 	for (my $i = 0; $i < $cpus; $i++) {threads->create('workerthread');}
 
 	# For each file against each other file
-	for (my $i = $startat; $i < scalar(@files); $i++) {
+	my $job_number = 0;
+	SPEC: for (my $i = 0; $i < scalar(@files)-1+$selfblast; $i++) {
 		for (my $j = $i+1-$selfblast; $j < scalar(@files); $j++) {
+			$job_number++;
+			if ($from != -1 && $job_number < $from) {next;}
 			# Wait for queque to get empty (with some buffer)
-			while ($jobque->pending() > 256 + 2*$cpus) {
-				sleep(1);
-			}
+			while ($jobque->pending() > 32 + 2*$cpus) {sleep(1);}
 			# Syncronize with other processes
 			$jobque->enqueue("$i $j");
+#			print STDERR "EN: $job_number -> $i $j\n";
+			if ($to != -1 && $job_number >= $to) {last SPEC;}
 		}
-		# Start-Stop
-		if ($stopat > -1 && $stopat <= $i) {last;}
 	}
-
 	# Tell all threads they can stop
 	{lock($all_jobs_submitted); $all_jobs_submitted = 1;}
 
 	# Wait until all jobs are done
 	foreach (threads->list()) 	{$_->join();}
 	&print_blast_stats();		print STDERR "\n";
-	print STDERR "-> Output written to $simgraph\n";
+	print STDERR "[OUTPUT] -> written to $simgraph\n";
 }
 
 sub workerthread {
@@ -423,7 +476,7 @@ sub workerthread {
 
 				{
 				lock($jobs_done);							# 	Productive
-				if ($jobs_done >= $jobs_todo) {						# 	Productive
+				if ($jobs_done >= $jobs_todo || ($part != -1 && $jobs_done >= $part)) {	# 	Productive
 #				lock($all_jobs_submitted);						# 	DEBUGGING
 #				if ($all_jobs_submitted) {						# 	DEBUGGING
 					if ($debug) {print STDERR "Thread $thread_id\tis leaving\n";}
@@ -459,6 +512,8 @@ sub workerthread {
 			$result_ji = &blast($file_j,$file_i);
 		}
 
+		if ($file_i eq $file_j && !$selfblast) {die("Selfblast is disabled but I want to check $file_i vs $file_j");}
+
 		my %lengths;
 		if ($file_i eq $file_j) {
 			# Get lengths once is enough (selfblast)
@@ -490,7 +545,7 @@ sub workerthread {
 			my %full_id_map_i;
 			my %full_id_map_j;
 			foreach (sort keys %reciprocal_matches) {
-				my ($a, $b) = split(" ",$_);
+				my ($a, $b) = split(/\s/,$_);
 				my $aa = &convertNCBI($a);
 				my $bb = &convertNCBI($b);
 				if ($aa ne $a) {
@@ -694,13 +749,13 @@ sub synteny_matches {
 	my @multis;	# array that contains all multi-edges
 	# Convert reciprocal matches to ffadj input
 	foreach (keys %reciprocal_matches) {
-		my @values = split(" ",$reciprocal_matches{$_});
-		my ($a, $b) = split(" ",$_);
+		my @values = split(/\s/,$reciprocal_matches{$_});
+		my ($a, $b) = split(/\s/,$_);
 		unless (defined($order{$a})) {$a = &convertNCBI($a);}
 		unless (defined($order{$b})) {$b = &convertNCBI($b);}
-		my @a = split(" ",$order{$a});
-		my @b = split(" ",$order{$b});
-		unless (defined($a[0])) {die();}
+		my @a = split(/\s/,$order{$a});
+		my @b = split(/\s/,$order{$b});
+		unless (defined($a[0])) {die("Failed parsing gene IDs from blast output/gff input\n");}
 
 		unless (defined($multis[$a[0]])) {$multis[$a[0]] = $b[0];}
 		else 				 {$multis[$a[0]] .= ','.$b[0];}
@@ -765,7 +820,14 @@ sub print_blast_stats {
 		lock($jobs_done);
 		my $percent = int($jobs_done/$jobs_todo*10000)/100;
 		print STDERR "\r                                                                               ";
-		print STDERR "\rRunning blast analysis: $percent% ($jobs_done/$jobs_todo)";
+	
+		if ($split_to_X_jobs == -1) {
+			print STDERR "\rRunning blast analysis: $percent% ($jobs_done/$jobs_todo)";
+		}
+		else {	# 5.16
+			$percent = int($jobs_done/$part*10000)/100;
+			print STDERR "\rRunning blast analysis: $percent% ($jobs_done/$part, $jobs_todo in total)";
+		}
 	}
 }
 
@@ -823,8 +885,10 @@ sub get_legal_matches {
 		if ($blastmode eq "tblastx+" || $blastmode eq "tblastx") {$alignment_length *= 3;}
 		if ($alignment_length < $length{$query_id}*($coverage/100)+0.5) 		{next;}
 		if ($alignment_length < $length{$subject_id}*($coverage/100)+0.5) 		{next;}
+
 		# It hit itself (only during selfblast)
-		if ($query_id eq $subject_id) 							{next;}
+		# if ($selfblast && $query_id eq $subject_id) 					{next;}	# 5.16 reuse IDs
+		## Listing them in the graph is okay, clustering will ignore them
 
 		# Similar hits? Take the better one
 		if (defined($result{"$query_id $subject_id"})) {
@@ -844,17 +908,22 @@ sub get_legal_matches {
 # Auto set the number of CPUs
 sub auto_cpus {
 	if ($cpus == 0) {
-		my $cpu_x = 0;
-		# Linux
-		if (-e "/proc/cpuinfo") {
-			$cpu_x = qx(grep processor /proc/cpuinfo | wc -l);
+		my $cpu_x = qx(getconf _NPROCESSORS_ONLN); 		$cpu_x =~ s/[^0-9]//g;
+		# Fallback
+		if (length($cpu_x) == 0 || $cpu_x == 0) {
+			# Linux
+			if (-e "/proc/cpuinfo") {
+				$cpu_x = qx(grep processor /proc/cpuinfo | wc -l);
+			}
+			# Try Mac
+			else {
+				$cpu_x = qx(system_profiler | grep CPUs:);
+			}
+			$cpu_x =~ s/[^0-9]//g;
 		}
-		# Try Mac
-		else {
-			$cpu_x = qx(system_profiler | grep CPUs:);
+		if (length($cpu_x) == 0 || $cpu_x == 0) {
+			print STDERR "failed! Use 1 core only\n";$cpu_x = 1;
 		}
-		$cpu_x =~ s/[^0-9]//g;
-		if (length($cpu_x) == 0 || $cpu_x == 0) {print STDERR "failed! Use 1 core only\n";$cpu_x = 1;}
 		print STDERR "Detected $cpu_x available CPU threads, ";
 		$cpus = $cpu_x;
 	}
@@ -892,8 +961,9 @@ sub blast {
 
 	# File does not exists yet or I am forced to rewrite it
 	if (!(-s $bla) || $force) {
-		if ($debug) {print STDERR "$command >$bla\n";}
-		system("$command | sort -k11,11g >'$bla'");		# run blast and presort (speeds up best alignment search but is NOT mandatory)
+		if ($debug) {print STDERR "$command >$bla.tmp\n";}											# 5.16
+		system("$command | sort -k11,11g >'$bla.tmp'");		# run blast and presort (speeds up best alignment search but is NOT mandatory)	# 5.16
+		system("mv '$bla.tmp' '$bla'");				# move when done, aids when job fails (no half bla files)			# 5.16
 		my @data = &readFile($bla);
 		unless ($keep) {unlink($bla);}				# delete tmp file
 		return \@data;
@@ -956,11 +1026,12 @@ sub check_blast {
 
 # Check plausibility of files
 sub check_files {
-	my %ids;
 	if (scalar(@files) < 2 && $step != 3)		{&print_usage; &Error("I need at least two files to compare something!");}
 	print STDERR "Checking input files\n";
 	foreach my $file (@files) {
-		&read_details($file,\%ids);
+		if ($verbose) {print STDERR "Checking $file... ";}
+		&read_details($file,1);
+		if ($verbose) {print STDERR "ok\n";}
 	}
 }
 
@@ -972,12 +1043,13 @@ sub convertNCBI {
 }
 
 sub read_details {
-	my %ids;
+	my %ids;				# local test for duplicated IDs
 	my %genes;
 	my $file = shift;
 	my $test = 0;
-	if (defined($_[0])) {$test = 1; %ids = %{(shift)};}	# if no ID Hash is give, we do not want to test but to fetch the gff data
+	if (defined($_[0])) {$test = 1;}	# if no ID Hash is give, we do not want to test but to fetch the gff data
 
+#	print STDERR "TEST: $test\n";
 	if (!-e $file)		{&Error("File '$file' not found!");}
 	open(FASTA,"<$file") || &Error("Could not open '$file': $!");
 	while (<FASTA>) {
@@ -987,7 +1059,8 @@ sub read_details {
 			$_ =~ s/^>//;
 			$_ =~ s/\s.*//;
 			if ($test) {
-				if (defined($ids{$_}))	{&Error("Gene ID '$_' is defined at least twice:\n$ids{$_}\n$file");}
+#				print STDERR "ID: $_\n";
+				if (defined($ids{$_}))	{&Error("Gene ID '$_' is defined at least twice in $file");}
 				$ids{$_} = $file;
 			}
 			if ($synteny) {
@@ -1001,10 +1074,11 @@ sub read_details {
 	unless ($synteny) {return;}
 
 	my %coordinates;
-	if ($verbose && $test) {print STDERR "$file\t".scalar(keys %genes)." genes\n";}
+	if ($verbose && $test) {print STDERR "$file\t".scalar(keys %genes)." genes\t";}
 	my $gff = &gff4fasta($file);
 	open(GFF,"<$gff") || &Error("Could not open '$gff': $!");
 	while (<GFF>) {
+		if ($_ =~ /^##FASTA/) {last;}	# deal with prokka gffs, thx 2 Ben Woodcroft
 		if ($_ =~ /^#/) {next;}
 		# e.g. NC_009925.1	RefSeq	CDS	9275	10096	.	-	0	ID=cds8;Name=YP_001514414.1;Parent=gene9;Dbxref=Genbank:YP_001514414.1,GeneID:5678848;gbkey=CDS;product=signal peptide peptidase SppA;protein_id=YP_001514414.1;transl_table=11
 		my @col = split(/\t+/,$_);
@@ -1033,7 +1107,7 @@ sub read_details {
 }
 
 sub Error {
-	print STDERR "Error: ".$_[0]."\n";
+	print STDERR "\n[ERROR] ".$_[0]."\n";
 	exit 0;
 }
 
@@ -1075,6 +1149,6 @@ sub write_descriptions {
 			}
 		}
 	}
-	print STDERR "-> Output written to $desctable\n";
+	print STDERR "[OUTPUT] -> written to $desctable\n";
 
 }
