@@ -3,7 +3,7 @@
  *	Reads edge list and splits connected components
  *	according to algebraic connectivity threshold
  *
- *	Last updated: 2016/04/22		
+ *	Last updated: 2017/09/20		
  *	Author: Marcus Lechner
  */
 
@@ -40,6 +40,8 @@ vector<unsigned int> get_deg_one (vector<unsigned int>& );
 // Parameters
 bool param_verbose 		= false;
 double param_con_threshold 	= 0.1;
+unsigned int debug_level	= 0;
+double param_sep_purity 	= 0.75;
 string param_rmgraph            = "remove.graph";
 unsigned int min_iter 		= 100;		// min number of alg con iterations
 unsigned int max_iter 		= 10000;	// max number of alg con iterations
@@ -98,6 +100,14 @@ int main(int argc, char *argv[]) {
 			paras++;
 			param_con_threshold = string2double(string(argv[paras]));
 		}
+		else if (parameter == "-purity") {
+			paras++;
+			param_sep_purity = string2double(string(argv[paras]));
+		}
+		else if (parameter == "-debug") {
+			paras++;
+			debug_level = int(string2double(string(argv[paras])));
+		}
 		else if (parameter == "-rmgraph") {
 			paras++;
 			param_rmgraph = string(argv[paras]);
@@ -109,9 +119,14 @@ int main(int argc, char *argv[]) {
 		}
 	}
 	
+	if (debug_level > 0) cerr << "[DEBUG] Debug level " << debug_level << endl;
 
 	// Parse files
-	for (vector<string>::iterator it=files.begin() ; it != files.end(); it++) parse_file(*it);
+	for (vector<string>::iterator it=files.begin() ; it != files.end(); it++) {
+		if (debug_level > 0) cerr << "[DEBUG] Parsing file " << *it << endl;
+		parse_file(*it);
+		if (debug_level > 0) cerr << "[DEBUG] I know " << species_counter <<  " species with " << protein_counter << " proteins and " << edges << " edges in sum" << endl;
+	};
 
 	// Free memory
 	files.clear();
@@ -122,6 +137,7 @@ int main(int argc, char *argv[]) {
 	if (param_verbose) cerr << species_counter << " species" << endl << protein_counter << " paired proteins" << endl << edges << " bidirectional edges" << endl;
 
 	// Prepare sort of output
+	if (debug_level > 0) cerr << "[DEBUG] Sorting known species" << endl;
 	sort_species();
 
 	// Write output header
@@ -131,6 +147,7 @@ int main(int argc, char *argv[]) {
 	graph_clean.open(param_rmgraph.c_str());
 
 	// Clustering
+	if (debug_level > 0) cerr << "[DEBUG] Clustering" << endl;
 	partition_graph();
 	graph_clean.close();
 	}
@@ -283,14 +300,19 @@ void partition_graph() {
 
 		// Do not report singles
 		if (current_group.size() < 2) {
+			if (debug_level > 1) cerr << "[DEBUG] Single gene skipped" << endl;
 			protein_id++;
 			continue;
 		}
 
 		// Connectivity analysis
+		if (debug_level > 0) cerr << "[DEBUG] Calculating connectivity of a group. " << current_group.size() << " proteins (ID: " << protein_id << ")" << endl;
+		if (current_group.size() > 16000) cerr << "[WARN] Current connected component contains " << current_group.size() << " proteins. That might be way to much for me. Try raising the e-value!" << endl;
 		double connectivity = getConnectivity(current_group);
+		if (debug_level > 0) cerr << "[DEBUG] Connectivity was " << connectivity << endl;
 
 		if (connectivity < param_con_threshold && current_group.size() > 3) {
+			if (debug_level > 0) cerr << "[DEBUG] Reiterating" << endl;
 			// Split groups is done in getConnectivity function
 			// Reset flags and repeat without incrementing protein counter
 			for (unsigned int i = 0; i < current_group.size(); i++) done[current_group[i]] = false;
@@ -432,7 +454,8 @@ void removeExternalEdges(map<unsigned int,bool>& a) {
 			for (vector<unsigned int>::iterator ita = graph[protein].edges.begin(); ita != graph[protein].edges.end(); ita++) {
 				// If it is not present the own group, set flag
 				if (a.find(*ita) == a.end()) {
-					graph_clean << graph[protein].full_name << "\t" << graph[*ita ].full_name << endl; // Improved graph cleaning
+					// 5.16 store name AND species in removal list
+					graph_clean << graph[protein].full_name << "\t" << species[graph[protein].species_id] << "\t" << graph[*ita ].full_name << "\t" << species[graph[*ita ].species_id] << endl; // Improved graph cleaning
 					swap = true;
 				}
 				// Otherwise, add it to the new edge list
@@ -497,37 +520,37 @@ void splitGroups(vector<double>& y, vector<unsigned int>& nodes){
 //		return;
 //	}
 
-	// Store data about two groups
-	map<unsigned int,bool> groupA, groupB;
+	// Store data about two groups (Zero cannot be assigned with certainty)
+	map<unsigned int,bool> groupA, groupB, groupZero;
 	for (unsigned int i = 0; i < y.size(); i++) {
-//		cerr << i << " = " << y[i] << endl;		
-		if (y[i] < 0) 		{groupA[nodes[i]] = true;} // cerr << graph[nodes[i]].full_name << " {color:#95cde5}" << endl; }
-		else              	{groupB[nodes[i]] = true;} // cerr << graph[nodes[i]].full_name << " {color:#b01700}" << endl; }
+		unsigned int style = 1;
+		if (y[i] < 0) style = 2;
+		if (abs(y[i]) < param_sep_purity) style = 0;
+		if (debug_level > 2) cerr << "[DEBUG L3] " << i << " = " << y[i] << " -> " << style << endl;
+		if      (abs(y[i]) < param_sep_purity)	{groupZero[nodes[i]] = true;} // cerr << graph[nodes[i]].full_name << " {color:#95cde5}" << endl; }
+		else if (y[i] < 0) 				{groupA[nodes[i]] = true;} // cerr << graph[nodes[i]].full_name << " {color:#95cde5}" << endl; }
+		else              				{groupB[nodes[i]] = true;} // cerr << graph[nodes[i]].full_name << " {color:#b01700}" << endl; }
 	}
 
 //	cerr << groupA.size() << " -- " << groupB.size() << endl;
 
 	// Catch error in laplacien calcs
-	if (groupA.size() == 0 || groupB.size() == 0) {
-		throw "Failed to partition subgraph! This might lead to an infinit loop. Please submit the .blastgraph file to lechner@staff.uni-marburg.de to help fixing this issue.";
+	if ((groupA.size() == 0 || groupB.size() == 0) && groupZero.size() == 0) {
+		throw string("Failed to partition subgraph! This might lead to an infinit loop. Please submit the .blastgraph file to lechner@staff.uni-marburg.de to help fixing this issue.");
 	}
 
+	removeExternalEdges(groupZero);
 	removeExternalEdges(groupA);
 	removeExternalEdges(groupB);
 }
 
 double getConnectivity(vector<unsigned int>& nodes) {
-	// Special cases hotfix
-	if (nodes.size() == 2) {return 1;}
-
-	if (nodes.size() == 3) {
-		vector<unsigned int> min = get_deg_one(nodes);
-		// fully connected
-		if (min.size() == 0) {return 1;}
-		// not
-		else {return 1/3;}
+	// Special case quicky
+	if (nodes.size() == 2) {
+		if (debug_level > 1) cerr << "[DEBUG L2] Shortcut for two-piece component -> conn = 1" << endl;
+		return 1;
 	}
-	// Hotfix end
+	// quicky end
 
 	// Get max degree of nodes
 	unsigned int max_degree = max_deg(nodes);
@@ -546,7 +569,9 @@ double getConnectivity(vector<unsigned int>& nodes) {
 
 	// Repeat until difference < epsilon
 	unsigned int iter = 0;							// catch huge clustering issues by keeping track here
+	
 	while(iter++ < max_iter) { 
+		if (debug_level > 2 && iter%100 == 0) cerr << "[DEBUG L2] Step " << iter << " / " << max_iter << endl;
 		last_length = current_length;
 		// Get a new x
 		x = get_new_x(norm, nodes, mapping);
@@ -557,9 +582,10 @@ double getConnectivity(vector<unsigned int>& nodes) {
 		// Get lenght (lambda) & normalize vector
 		norm = nomalize(x_hat, &current_length);
 ///		cerr << "I " << iter << ": " << abs(current_length-last_length) << endl; 
-		if (abs(current_length-last_length) < 0.00001 && iter >= min_iter) break;	// min 100 iterations (prevent convergence by chance), converge to 1e-6
+		if (abs(current_length-last_length) < 0.0001 && iter >= min_iter) break;	// min 100 iterations (prevent convergence by chance), converge to 1e-6
 	}
 ///		cerr << nodes.size() << " nodes done after " << iter << " iterations" << endl;
+	if (debug_level > 0) cerr << "[DEBUG] " << iter << " / " << max_iter << " iterations required (error is " << abs(current_length-last_length) << ")" << endl;
 
 	double connectivity = (-current_length+2*max_degree)/(nodes.size());
 
@@ -595,7 +621,7 @@ void remove_edge(protein& node, const unsigned int remove_id) {
 	// Search for element in edge list
 	vector<unsigned int>::iterator element = find(node.edges.begin(), node.edges.end(), remove_id);
 	// Not found (something is wrong!)
-	if (element == node.edges.end()) throw "Element could not be found in edge list";
+	if (element == node.edges.end()) throw string("Element could not be found in edge list");
 //	// Remove element
 	node.edges.erase(element);
 }
@@ -622,13 +648,13 @@ void parse_file(string file) {
 
 				if (file_a == "file_a" && file_b == "file_b") continue;	// Init Header
 
-				// Map species a
+				// Map species a if not know so far
 				if (species2id.find(file_a) == species2id.end())	{
 						species.push_back(file_a);
 //						cerr << "Species " << species_counter << ": " << file_a << endl;
 						species2id[file_a] = species_counter++;
 				}
-				// Map species b
+				// Map species b if not know so far
 				if (species2id.find(file_b) == species2id.end())	{
 //						cerr << "Species " << species_counter << ": " << file_b << endl;
 						species.push_back(file_b);
@@ -644,17 +670,27 @@ void parse_file(string file) {
 
 //				cerr << protein_counter << ": " << fields[0] << " <-> " << fields[1] << endl;
 
+				// 5.16 deal with duplicated IDs by adding file ID to protein ID
+				string ida = fields[0];
+				string idb = fields[1];
+				fields[0] += " "; fields[0] += file_a_id;
+				fields[1] += " "; fields[1] += file_b_id;
+
+				// 5.16 do not point to yourself
+				if (!fields[0].compare(fields[1])) {continue;}
+
 				// A new protein
 				if (protein2id.find(fields[0]) == protein2id.end())	{
 					protein a;
-					a.full_name	= fields[0];
+					a.full_name	= ida;
 					a.species_id	= file_a_id;
 					protein2id[fields[0]] = protein_counter++;
 					graph.push_back(a);
 				}
 				if (protein2id.find(fields[1]) == protein2id.end())	{
 					protein b;
-					b.full_name	= fields[1];	b.species_id	= file_b_id;
+					b.full_name	= idb;
+					b.species_id	= file_b_id;
 					protein2id[fields[1]] = protein_counter++;
 					graph.push_back(b);
 				}
@@ -670,7 +706,7 @@ void parse_file(string file) {
 		}
 	}
 	else {
-		throw "Could not open file " + file;
+		throw string("Could not open file ") + file;
 	}
 }
 
